/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : May. 2006

 File          : main.c, SPI_Functions.c, SPI_Functions.h

 Hardware      : ADuC702x Rev I silicon, Spansion S25FL032A
                  ADuC702x        ->      S25FL032A
                  P1.4  ( SCLK )          SCK   ( Pin 16 )
                  P1.5  ( MISO )          SO    ( Pin 8 )
                  P1.6  ( MOSI )          SI    ( Pin 15 )
                  P1.7  ( CS )            CS#   ( Pin 7 )
                  P3.0                    W#    ( Pin 9 )
                  P3.1                    Hold# ( Pin 1 )

 Description   : Example program for integration of Spansion S25FL032A
                 SPI Flash Module.

                 This file contains Example SPI functions used in main.c

*********************************************************************/

#include "ioADuC7026.h"

// Place Function prototypes here
unsigned long Read_Identification(void);
unsigned char Read_Status_Register(void);
void Write_Status_Register(unsigned char c_data);
void Write_Enable(void);
void Write_Disable(void);
void Bulk_Erase(void);
void Page_Program(unsigned long l_address, unsigned char *c_data, unsigned long Number_Bytes);
void Data_Read(unsigned long l_address, unsigned char *c_data, unsigned long Number_Bytes);
void Sector_Erase(unsigned long l_address);


/*********************************************************************

 Author         : ADI - Apps            www.analog.com/MicroConverter

 Date           : May. 2006

 Function Name  : void Sector_Erase(unsigned long l_address)

 Arguement      : l_address.

 Description    : This function erases the sector pointed to by l_address

 Note           : This function does not return until command is completed

*********************************************************************/

void Sector_Erase(unsigned long l_address)
{
volatile  unsigned char  SPI_DATA ; // Dummy variable

  Write_Enable();           // Enable Write in Status register

  GP1DAT    =   0x80000000; // Drive CS low

  SPITX = 0x0D8;            // Transmit command for sector erase
  while((SPISTA&0x2) == 0 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

                            // Tranmit bits 24-16 of the address
  SPITX = ( l_address & 0xFF0000 ) >>16;
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read
                            // Tranmit bits 15-8 of the address
  SPITX = ( l_address & 0xFF00 ) >> 8;
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

                            // Tranmit bits 7-0 of the address
  SPITX = ( l_address & 0xFF );
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  GP1DAT    =   0x80800000; // Drive CS High

                            // Wait until command is completed
  while( Read_Status_Register() & 0x1 )
  {}

  Write_Disable();          // Disable write access via Status register

}

/*********************************************************************

 Author         : ADI - Apps            www.analog.com/MicroConverter

 Date           : May. 2006

 Function Name  : void Data_Read(unsigned long l_address, unsigned char *c_data, unsigned long Number_Bytes)

 Arguement      : l_address,*c_data,Number_Bytes

 Description    : This function reads the number of bytes specifiec in Number_Bytes,
                  from the address pointed to by l_address and places the information
                  in c_data.

*********************************************************************/

void Data_Read(unsigned long l_address, unsigned char *c_data, unsigned long Number_Bytes)
{
volatile  unsigned char  SPI_DATA ; // Dummy variable
  int i;

  GP1DAT    =   0x80000000; // Drive CS low

  SPITX = 0x03;             // Transmit command to read data
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  SPITX = ( l_address & 0xFF0000 ) >>16;
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  SPITX = ( l_address & 0xFF00 ) >> 8;
    while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  SPITX = ( l_address & 0xFF );
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  for ( i = 0; i < Number_Bytes ; i++ )
  {
    SPITX = 0x00 ;            // Dummy Transmit
    while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
    {}
    while((SPISTA&0x8) == 0 ) // Wait until RX register is full
    {}

    *c_data = SPIRX;          // Read Data
    *c_data ++ ;              // Increment pointer
  }

  GP1DAT    =   0x80800000; // Drive CS High

}

/*********************************************************************

 Author         : ADI - Apps            www.analog.com/MicroConverter

 Date           : May. 2006

 Function Name  : void Page_Program(unsigned long l_address, unsigned char *c_data, unsigned long Number_Bytes)

 Arguement      : l_address,*c_data,Number_Bytes

 Description    : This function programs the number of bytes specified in Number_Bytes,
                  to the address pointed to by l_address. The data uses is c_data.

 Note           : This function does not return until command is completed

*********************************************************************/

void Page_Program(unsigned long l_address, unsigned char *c_data, unsigned long Number_Bytes)
{
volatile  unsigned char  SPI_DATA ; // Dummy variable
  int i;

  Write_Enable();


  GP1DAT    =   0x80000000; // Drive CS low

  SPITX = 0x02;             // Transmit command to program page
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  SPITX = ( l_address & 0xFF0000 ) >>16;
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  SPITX = ( l_address & 0xFF00 ) >> 8;
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

 SPITX = ( l_address & 0xFF );
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  for ( i = 0; i < Number_Bytes ; i++ )
  {
                            // Transmit data
    SPITX = ( *c_data  & 0xFF );
    while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
    {}
    while((SPISTA&0x8) == 0 ) // Wait until RX register is full
    {}
    SPI_DATA = SPIRX;         // Dummy Read
    *c_data ++ ;              // Increment pointer
  }

  GP1DAT    =   0x80800000; // Drive CS High

                            // Wait until command is completed
  while( Read_Status_Register() & 0x1 )
  {}
  Write_Disable();          // Disable write access via Status register

}

/*********************************************************************

 Author         : ADI - Apps            www.analog.com/MicroConverter

 Date           : May. 2006

 Function Name  : void Bulk_Erase(void)

 Arguement      :

 Description    : This function erases the entire Flash.

 Note           : This function does not return until command is completed

*********************************************************************/

void Bulk_Erase(void)
{
volatile  unsigned char  SPI_DATA ; // Dummy variable

  Write_Enable();           // Enable write access via Status register


  GP1DAT    =   0x80000000; // Drive CS low

  SPITX = 0xC7;             // Transmit Bulk Erase command
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  GP1DAT    =   0x80800000; // Drive CS High

                            // Wait until command is completed
  while( Read_Status_Register() & 0x1 )
  {}
  Write_Disable();          // Disable write access via Status register
}

/*********************************************************************

 Author         : ADI - Apps            www.analog.com/MicroConverter

 Date           : May. 2006

 Function Name  : void Write_Disable(void)

 Arguement      :

 Description    : This function disables write access to the Flash.

 Note           : This function does not return until command is completed

*********************************************************************/

void Write_Disable(void)
{
volatile  unsigned char  SPI_DATA ; // Dummy variable

  GP1DAT    =   0xF0000000; // Drive CS low

  SPITX = 0x4;              // Transmit command to Disable write access
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read
  GP1DAT    =   0xF0800000; // Drive CS High

                            // Wait until command is completed
  while( Read_Status_Register() & 0x1 )
  {}
}

/*********************************************************************

 Author         : ADI - Apps            www.analog.com/MicroConverter

 Date           : May. 2006

 Function Name  : void Write_Enable(void)

 Arguement      :

 Description    : This function enables write access to the Flash.

 Note           : This function does not return until command is completed

*********************************************************************/

void Write_Enable(void)
{
volatile  unsigned char  SPI_DATA ; // Dummy variable

  GP1DAT    =   0x80000000; // Drive CS low

  SPITX = 0x6;              // Transmit command to Enable Write access
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read
  GP1DAT    =   0x80800000; // Drive CS High

                            // Wait until command is completed
  while( Read_Status_Register() & 0x1 )
  {}
}

/*********************************************************************

 Author         : ADI - Apps            www.analog.com/MicroConverter

 Date           : May. 2006

 Function Name  : void Write_Status_Register(unsigned char c_data)

 Arguement      : c_data

 Description    : This function writes to the status register.

 Note           : This function does not return until command is completed

*********************************************************************/

void Write_Status_Register(unsigned char c_data)
{
volatile  unsigned char  SPI_DATA ; // Dummy variable

  Write_Enable();

  GP1DAT    =   0x80000000; // Drive CS low

  SPITX = 0x1;              // Transmit command to write to Status Register
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  SPITX = c_data;           // Transmit Data
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  GP1DAT    =   0x80800000; // Drive CS High

                            // Wait until command is completed
  while( Read_Status_Register() & 0x1 )
  {}
  Write_Disable();          // Disable write access via Status register
}

/*********************************************************************

 Author         : ADI - Apps            www.analog.com/MicroConverter

 Date           : May. 2006

 Function Name  : unsigned char Read_Status_Register(void)

 Arguement      :

 Description    : This function reads the status register.

*********************************************************************/

unsigned char Read_Status_Register(void)
{
  unsigned char   SPI_DATA ;

  GP1DAT    =   0x80000000; // Drive CS low

  SPITX = 0x5;              // Transmit command to Read the Status Register
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  SPITX = 0x00;             // Dummy Transmit
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  GP1DAT    =   0x80800000; // Drive CS High

  return ( SPI_DATA );

}


/*********************************************************************

 Author         : ADI - Apps            www.analog.com/MicroConverter

 Date           : May. 2006

 Function Name  : unsigned long Read_Identification()

 Arguement      :

 Description    : This function reads the parts identification number.
                  For a full description, please refer to the S25FL0XX Datasheet

*********************************************************************/

unsigned long Read_Identification()
{
  unsigned long   SPI_DATA ;

  GP1DAT    =   0x80000000; // Drive CS low

  SPITX = 0x9F;             // Transmit command to read the ID
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX;         // Dummy Read

  SPITX = 0x00;
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA = SPIRX<<16;

  SPITX = 0x00;
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA += SPIRX<<8;

  SPITX = 0x00;
  while((SPISTA&0x1) == 1 ) // Wait until command is transmitted
  {}
  while((SPISTA&0x8) == 0 ) // Wait until RX register is full
  {}
  SPI_DATA += SPIRX;

  GP1DAT    =   0x80800000; // Drive CS High

  return ( SPI_DATA );

}
